@@ -22,16 +22,8 @@ sub new () {
     my ( $class, @args ) = @_;
     my $self = $class->SUPER::new(@args);
 
-    $self->{'_Class'} =
-      ref($self);    # Cache it since we're gonna look at it a _lot_
-
-    if ( $self->can('_CacheConfig') ) {
-        $self->{'_CacheConfig'} = $self->_CacheConfig();
-    }
-    else {
-        $self->{'_CacheConfig'} = __CachableDefaults::_CacheConfig();
-    }
-
+    # Cache it since we're gonna look at it a _lot_
+    $self->{'_Class'} = ref($self);
 
     return ($self);
 }
@@ -40,9 +32,22 @@ sub _SetupCache {
     my $self  = shift;
     my $cache = shift;
     $_CACHES{$cache} = Cache::Simple::TimedExpiry->new();
-    $_CACHES{$cache}->expire_after( $self->{'_CacheConfig'}{'cache_for_sec'} );
+    $_CACHES{$cache}->expire_after( $self->_CacheConfig->{'cache_for_sec'} );
 }
 
+
+=head2 FlushCache 
+
+This class method flushes the _global_ DBIx::SearchBuilder::Record::Cachable 
+cache.  All caches are immediately expired.
+
+=cut
+
+sub FlushCache {
+    %_CACHES = ();
+}
+
+
 sub _KeyCache {
     my $self = shift;
     my $cache =     $self->_Handle->DSN . "-KEYS--" . $self->{'_Class'};
@@ -119,7 +124,7 @@ sub LoadByCols {
 sub __Set () {
     my ( $self, %attr ) = @_;
 
-    $self->_expire( $self->_primary_RecordCache_key() );
+    $self->_expire();
     return $self->SUPER::__Set(%attr);
 
 }
@@ -129,12 +134,12 @@ sub __Set () {
 # Args    : nil
 # Lvalue  : ?
 
-sub Delete () {
+sub __Delete () {
     my ($self) = @_;
 
-    $self->_expire( $self->_primary_RecordCache_key() );
+    $self->_expire();
 
-    return $self->SUPER::Delete();
+    return $self->SUPER::__Delete();
 
 }
 
@@ -158,15 +163,12 @@ sub _expire (\$) {
 sub _fetch () {
     my ( $self, $cache_key ) = @_;
     my $data = $self->_RecordCache->fetch($cache_key) or return;
+
     @{$self}{keys %$data} = values %$data; # deserialize
     return 1;
 
 }
 
-sub id {
-    my $self = shift;
-    return ( $self->{'_id'} ||= $self->SUPER::id );
-}
 
 sub __Value {
     my $self  = shift;
@@ -205,7 +207,8 @@ sub _serialize {
 
 sub _gen_alternate_RecordCache_key {
     my ( $self, %attr ) = @_;
-    my $cache_key = $self->Table() . ':';
+    #return( Storable::nfreeze( %attr));
+   my $cache_key;
     while ( my ( $key, $value ) = each %attr ) {
         $key   ||= '__undef';
         $value ||= '__undef';
@@ -229,7 +232,7 @@ sub _gen_alternate_RecordCache_key {
 
 sub _fetch_RecordCache_key {
     my ($self) = @_;
-    my $cache_key = $self->{'_CacheConfig'}{'cache_key'};
+    my $cache_key = $self->_CacheConfig->{'cache_key'};
     return ($cache_key);
 }
 
@@ -285,8 +288,6 @@ sub _lookup_primary_RecordCache_key {
 
 }
 
-package __CachableDefaults;
-
 sub _CacheConfig {
     {
         'cache_p'       => 1,
